@@ -95,6 +95,7 @@ void cleanup_for_exec ();
 void *palloc(struct pool *, int nbytes);
 void *pcalloc(struct pool *, int nbytes);
 extern char *pstrdup(struct pool *, const char *s);
+extern char *pstrndup(struct pool *, const char *s, int n);
 char *pstrcat(struct pool *, ...); /* all '...' must be char* */
 
 /* array and alist management... keeping lists of things.
@@ -144,8 +145,10 @@ typedef struct {
 table *make_table (pool *p, int nelts);
 table *copy_table (pool *p, table *);     
 char *table_get (table *, char *);
-void table_set (table *, char *name, char *val);
+void table_set (table *, const char *name, const char *val);
 void table_merge (table *, char *name, char *more_val);
+void table_unset (table *, char *key);
+void table_add (table *, char *name, char *val);
 
 table *overlay_tables (pool *p, table *overlay, table *base);     
 
@@ -224,6 +227,10 @@ enum kill_conditions { kill_never, kill_always, kill_after_timeout, just_wait};
 
 int spawn_child (pool *, void (*)(void *), void *,
 		 enum kill_conditions, FILE **pipe_in, FILE **pipe_out);
+#ifdef __EMX__
+int spawn_child_os2 (pool *, void (*)(void *), void *,
+         enum kill_conditions, FILE **pipe_in, FILE **pipe_out, char *buffer, int lenp);
+#endif
 
 /* magic numbers --- only one so far, min free bytes in a new pool block */
 
