@@ -264,17 +264,12 @@ int check_auth (request_rec *r) {
    return run_method (r, XtOffsetOf (module, auth_checker), 0);
 }
 
-/*
- * This should really do multiple passes, choosing wildcards last,
- * which would allow us to declare default_handler as a wildcard on
- * core_module.  Close enough for now...
- */
-
 int invoke_handler (request_rec *r)
 {
    module *modp;
    handler_rec *handp;
    char *content_type = r->content_type ? r->content_type : default_type (r);
+   char *handler = r->handler ? r->handler : content_type;
   
    /* Pass one --- direct matches */
    
@@ -283,7 +278,7 @@ int invoke_handler (request_rec *r)
        if (!modp->handlers) continue;
        
        for (handp = modp->handlers; handp->content_type; ++handp) {
-	   if (!strcasecmp (content_type, handp->content_type)) {
+	   if (!strcasecmp (handler, handp->content_type)) {
 	       int result = (*handp->handler)(r);
 
 	       if (result != DECLINED) return result;
@@ -305,7 +300,7 @@ int invoke_handler (request_rec *r)
 
 	   len = starp - handp->content_type;
 	   
-	   if (!len || !strncasecmp (content_type, handp->content_type, len))
+	   if (!len || !strncasecmp (handler, handp->content_type, len))
 	   {
 	       int result = (*handp->handler)(r);
 
@@ -423,7 +418,7 @@ char *invoke_cmd(command_rec *cmd, cmd_parms *parms, void *mconfig, char *args)
 	    return pstrcat (parms->pool, cmd->name, " must be On or Off",
 			    NULL);
 
-	return (*cmd->func) (parms, mconfig, strcasecmp (w, "off"));
+	return (*cmd->func) (parms, mconfig, strcasecmp (w, "off") != 0);
 
     default:
 
@@ -526,7 +521,12 @@ cmd_parms default_parms = { NULL, 0, -1, NULL, 0, NULL, NULL, NULL, NULL };
 
 char *server_root_relative (pool *p, char *file)
 {
+#ifdef __EMX__
+    /* Add support for OS/2 drive names */
+    if ((file[0] == '/') || (file[1] == ':')) return file;
+#else
     if (file[0] == '/') return file;
+#endif    
     return make_full_path (p, server_root, file);
 }
 
@@ -575,6 +575,17 @@ int parse_htaccess(void **result, request_rec *r, int override,
     FILE *f;
     cmd_parms parms;
     char *errmsg;
+    const struct htaccess_result *cache;
+    struct htaccess_result *new;
+    void *dc;
+
+/* firstly, search cache */
+    for (cache=r->htaccess; cache != NULL; cache=cache->next)
+	if (cache->override == override && strcmp(cache->dir, d) == 0)
+	{
+	    if (cache->htaccess != NULL) *result = cache->htaccess;
+	    return OK;
+	}
 
     parms = default_parms;
     parms.override = override;
@@ -584,7 +595,7 @@ int parse_htaccess(void **result, request_rec *r, int override,
     parms.path = d;
 
     if((f=pfopen(r->pool, filename, "r"))) {
-        void *dc = create_per_dir_config (r->pool);
+        dc = create_per_dir_config (r->pool);
 	
         parms.infile = f;
 	parms.config_file = filename;
@@ -599,10 +610,19 @@ int parse_htaccess(void **result, request_rec *r, int override,
 	}
 	
 	*result = dc;
-	return OK;
-    }
-    else
-        return OK;
+    } else
+	dc = NULL;
+
+/* cache it */
+    new = palloc(r->pool, sizeof(struct htaccess_result));
+    new->dir = pstrdup(r->pool, d);
+    new->override = override;
+    new->htaccess = dc;
+/* add to head of list */
+    new->next = r->htaccess;
+    r->htaccess = new;
+
+    return OK;
 }
 
 /*****************************************************************
@@ -626,17 +646,22 @@ server_rec *init_virtual_host (pool *p, char *hostname)
     }
 #endif
 
-    s->port = 0;
     s->server_admin = NULL;
     s->server_hostname = NULL; 
     s->error_fname = NULL;
     s->srm_confname = NULL;
     s->access_confname = NULL;
     s->timeout = 0;
-    s->do_rfc931 = 0;
-    s->host_addr.s_addr = get_virthost_addr (hostname, 0);
+    s->keep_alive_timeout = 0;
+    s->keep_alive = -1;
+    s->host_addr.s_addr = get_virthost_addr (hostname, &s->host_port);
+    s->port = s->host_port;  /* set them the same, by default */
     s->next = NULL;
 
+    s->is_virtual = 1;
+    s->virthost = pstrdup(p, hostname);
+    s->names = NULL;
+
     s->module_config = create_empty_config (p);
     s->lookup_defaults = create_per_dir_config (p);
     
@@ -645,7 +670,7 @@ server_rec *init_virtual_host (pool *p, char *hostname)
 
 int is_virtual_server (server_rec *s)
 {
-    return s->host_addr.s_addr != htonl (INADDR_ANY);
+    return s->is_virtual;
 }
 
 void fixup_virtual_hosts (pool *p, server_rec *main_server)
@@ -675,8 +700,11 @@ void fixup_virtual_hosts (pool *p, server_rec *main_server)
 	if (virt->timeout == 0)
 	    virt->timeout = main_server->timeout;
 
-	if (virt->do_rfc931 == 0)
-	    virt->do_rfc931 = main_server->do_rfc931;
+	if (virt->keep_alive_timeout == 0)
+	    virt->keep_alive_timeout = main_server->keep_alive_timeout;
+
+	if (virt->keep_alive == -1)
+	    virt->keep_alive = main_server->keep_alive;
     }
 }
 
@@ -696,11 +724,12 @@ void init_config_globals (pool *p)
     daemons_to_start = DEFAULT_START_DAEMON;
     daemons_min_free = DEFAULT_MIN_FREE_DAEMON;
     daemons_max_free = DEFAULT_MAX_FREE_DAEMON;
-    daemons_limit = DEFAULT_SERVER_LIMIT;
+    daemons_limit = HARD_SERVER_LIMIT;
     pid_fname = DEFAULT_PIDLOG;
     scoreboard_fname = DEFAULT_SCOREBOARD;
     max_requests_per_child = DEFAULT_MAX_REQUESTS_PER_CHILD;
     bind_address.s_addr = htonl(INADDR_ANY);
+    listeners = NULL;
 }
 
 server_rec *init_server_config(pool *p)
@@ -714,12 +743,14 @@ server_rec *init_server_config(pool *p)
     s->srm_confname = RESOURCE_CONFIG_FILE;
     s->access_confname = ACCESS_CONFIG_FILE;
     s->timeout = DEFAULT_TIMEOUT;
-    s->do_rfc931 = DEFAULT_RFC931;
+    s->keep_alive_timeout = DEFAULT_KEEPALIVE_TIMEOUT;
+    s->keep_alive = DEFAULT_KEEPALIVE;
     s->next = NULL;
     s->host_addr.s_addr = htonl (INADDR_ANY); /* NOT virtual host;
 					       * don't match any real network
 					       * interface.
 					       */
+    s->host_port = 0; /* matches any port */
 
     s->module_config = create_server_config (p, s);
     s->lookup_defaults = create_default_per_dir_config (p);
