@@ -51,12 +51,6 @@
  *
  */
 
-#ifdef WWW_USER
-#include <sys/param.h>
-#include <sys/types.h>
-#include <pwd.h>
-#include <string.h>
-#endif
 
 #define CORE_PRIVATE
 #include "httpd.h"
@@ -140,35 +134,7 @@ void *create_core_server_config (pool *a, server_rec *s)
     int is_virtual = s->is_virtual;
   
     conf->access_name = is_virtual ? NULL : DEFAULT_ACCESS_FNAME;
-
-    /*
-     * The root of document is either the home of the WWW_USER or
-     * what was defined in the configuration.
-     *
-     */
-
-    if (is_virtual) {
-	conf->document_root = 0;
-    } else {
-#ifdef WWW_USER
-	struct passwd* www_user = getpwnam(WWW_USER);
-	if (www_user && *www_user->pw_dir) {
-	    static char www_htdocs[MAXPATHLEN + 1];
-
-	    strcpy(www_htdocs, www_user->pw_dir);
-#ifdef WWW_USER_DOCUMENT_LOCATION
-	    strcat(strcat(www_htdocs, "/"), WWW_USER_DOCUMENT_LOCATION);
-#endif
-
-	    conf->document_root = www_htdocs;
-	} else {
-	    conf->document_root = DOCUMENT_LOCATION;
-	}
-#else
-	conf->document_root = DOCUMENT_LOCATION;
-#endif
-    }
-
+    conf->document_root = is_virtual ? NULL : DOCUMENT_LOCATION;
     conf->sec = make_array (a, 40, sizeof(void *));
     conf->sec_url = make_array (a, 40, sizeof(void *));
     
