@@ -1,6 +1,6 @@
 
 /* ====================================================================
- * Copyright (c) 1995, 1996 The Apache Group.  All rights reserved.
+ * Copyright (c) 1995-1997 The Apache Group.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -119,15 +119,18 @@ typedef struct {
 void make_cookie(request_rec *r)
 {
     struct timeval tv;
-    char new_cookie[100];	/* blurgh */
+    char new_cookie[1024];	/* blurgh */
     char *dot;
-    const char *rname = pstrdup(r->pool, 
+    char *rname = pstrdup(r->pool, 
 				get_remote_host(r->connection, r->per_dir_config,
 						REMOTE_NAME));
     
     struct timezone tz = { 0 , 0 };
 
     if ((dot = strchr(rname,'.'))) *dot='\0';	/* First bit of hostname */
+    if (strlen (rname) > 255)
+      rname[256] = 0;
+
     gettimeofday(&tv, &tz);
     sprintf(new_cookie,"%s%s%d%ld%d; path=/",
         COOKIE_NAME, rname,
