@@ -50,26 +50,6 @@
  *
  */
 
-/*
- * The @ and # characters in the following comment are used by the Debian
- * package creation procedure to automatically extract a short description
- * and a full doc. for the module (the lines between # (with the @ lines
- * removed) go to the short description, the lines between @ (without any
- * of the # lines) go to the doc).
- *
- */
-
-/*
- * @
- * #
- * mod_proxy: experimental server proxy support
- * #
- *
- * See the Apache documentation for usage.
- * @
- */
-
-#include "httpd.h"
 /*
 Note that the Explain() stuff is not yet complete.
 Also note numerous FIXMEs and CHECKMEs which should be eliminated.
