@@ -58,6 +58,7 @@
 #include "http_log.h"
 #include "http_protocol.h"
 #include "http_core.h"		/* For document_root.  Sigh... */
+#include "http_request.h"       /* for sub_req_lookup_uri() */
 #include "util_script.h"
 
 /*
@@ -66,6 +67,9 @@
  * in one place (if only to avoid creating inter-module dependancies
  * where there don't have to be).
  */
+ 
+#define MALFORMED_MESSAGE "malformed header from script. Bad header="
+#define MALFORMED_HEADER_LENGTH_TO_SHOW 30
 
 char **create_argv(pool *p, char *av0, char *args) {
     register int x,n;
@@ -79,7 +83,7 @@ char **create_argv(pool *p, char *av0, char *args) {
     av[0] = av0;
 
     for(x=1;x<n;x++) {
-        w = getword(p, &args, '+');
+        w = getword_nulls(p, &args, '+');
         unescape_url(w);
         av[x] = escape_shell_cmd(p, w);
     }
@@ -124,6 +128,7 @@ void add_common_vars(request_rec *r)
     table *e = r->subprocess_env;
     server_rec *s = r->server;
     conn_rec *c = r->connection;
+    const char *rem_logname;
     
     char port[40],*env_path;
     
@@ -163,7 +168,8 @@ void add_common_vars(request_rec *r)
     table_set (e, "SERVER_SOFTWARE", SERVER_VERSION);
     table_set (e, "SERVER_NAME", s->server_hostname);
     table_set (e, "SERVER_PORT", port);
-    table_set (e, "REMOTE_HOST", c->remote_name);
+    table_set (e, "REMOTE_HOST",
+	       get_remote_host(c, r->per_dir_config, REMOTE_NAME));
     table_set (e, "REMOTE_ADDR", c->remote_ip);
     table_set (e, "DOCUMENT_ROOT", document_root(r)); /* Apache */
     table_set (e, "SERVER_ADMIN", s->server_admin); /* Apache */
@@ -171,7 +177,8 @@ void add_common_vars(request_rec *r)
     
     if (c->user) table_set(e, "REMOTE_USER", c->user);
     if (c->auth_type) table_set(e, "AUTH_TYPE", c->auth_type);
-    if (c->remote_logname) table_set(e, "REMOTE_IDENT", c->remote_logname);
+    rem_logname = get_remote_logname(r);
+    if (rem_logname) table_set(e, "REMOTE_IDENT", rem_logname);
     
     /* Apache custom error responses. If we have redirected set two new vars */
     
@@ -181,6 +188,57 @@ void add_common_vars(request_rec *r)
     }
 }
 
+
+void add_cgi_vars(request_rec *r)
+{
+    table *e = r->subprocess_env;
+
+    table_set (e, "GATEWAY_INTERFACE","CGI/1.1");
+    table_set (e, "SERVER_PROTOCOL", r->protocol);
+    table_set (e, "REQUEST_METHOD", r->method);
+    table_set (e, "QUERY_STRING", r->args ? r->args : "");
+    
+    /* Note that the code below special-cases scripts run from includes,
+     * because it "knows" that the sub_request has been hacked to have the
+     * args and path_info of the original request, and not any that may have
+     * come with the script URI in the include command.  Ugh.
+     */
+    
+    if (!r->path_info || !*r->path_info || !strcmp (r->protocol, "INCLUDED")) {
+        table_set (e, "SCRIPT_NAME", r->uri);
+    } else {
+        int path_info_start = strlen (r->uri) - strlen (r->path_info);
+	
+	r->uri[path_info_start] = '\0';
+	table_set (e, "SCRIPT_NAME", r->uri);
+	r->uri[path_info_start] = '/';
+    }
+	
+    if (r->path_info && r->path_info[0]) {
+	/*
+ 	 * To get PATH_TRANSLATED, treat PATH_INFO as a URI path.
+	 * Need to re-escape it for this, since the entire URI was
+	 * un-escaped before we determined where the PATH_INFO began.
+ 	 */
+	request_rec *pa_req = sub_req_lookup_uri(
+				    escape_uri(r->pool, r->path_info), r);
+      
+        table_set (e, "PATH_INFO", r->path_info);
+
+	/* Don't bother destroying pa_req --- it's only created in
+	 * child processes which are about to jettison their address
+	 * space anyway.  BTW, we concatenate filename and path_info
+	 * from the sub_request to be compatible in case the PATH_INFO
+	 * is pointing to an object which doesn't exist.
+	 */
+	
+	if (pa_req->filename)
+	    table_set (e, "PATH_TRANSLATED",
+		       pstrcat (r->pool, pa_req->filename, pa_req->path_info,
+				NULL));
+    }
+}
+
 int scan_script_header(request_rec *r, FILE *f)
 {
     char w[MAX_STRING_LEN];
@@ -192,7 +250,7 @@ int scan_script_header(request_rec *r, FILE *f)
     while(1) {
 
 	if (fgets(w, MAX_STRING_LEN-1, f) == NULL) {
-	    log_reason ("malformed header from script", r->filename, r);
+	    log_reason ("Premature end of script headers", r->filename, r);
 	    return SERVER_ERROR;
         }
 
@@ -213,12 +271,15 @@ int scan_script_header(request_rec *r, FILE *f)
 	/* if we see a bogus header don't ignore it. Shout and scream */
 	
         if(!(l = strchr(w,':'))) {
+	    char malformed[(sizeof MALFORMED_MESSAGE)+1+MALFORMED_HEADER_LENGTH_TO_SHOW];
+            strcpy(malformed, MALFORMED_MESSAGE);
+            strncat(malformed, w, MALFORMED_HEADER_LENGTH_TO_SHOW);
             /* Soak up all the script output --- may save an outright kill */
 	    while (fgets(w, MAX_STRING_LEN-1, f) != NULL)
 	        continue;
 	    
 	    kill_timeout (r);
-	    log_reason ("malformed header from script", r->filename, r);
+	    log_reason (malformed, r->filename, r);
 	    return SERVER_ERROR;
         }
 
@@ -238,22 +299,66 @@ int scan_script_header(request_rec *r, FILE *f)
             sscanf(l, "%d", &r->status);
             r->status_line = pstrdup(r->pool, l);
         }
+        else if(!strcasecmp(w,"Location")) {
+	    table_set (r->headers_out, w, l);
+        }   
+
+/* The HTTP specification says that it is legal to merge duplicate
+ * headers into one.  Some browsers that support Cookies don't like
+ * merged headers and prefer that each Set-Cookie header is sent
+ * separately.  Lets humour those browsers.
+ */
+	else if(!strcasecmp(w, "Set-Cookie")) {
+	    table_add(r->err_headers_out, w, l);
+	}
         else {
-	    table_merge (r->headers_out, w, l);
+	    table_merge (r->err_headers_out, w, l);
         }
     }
 }
 
 void send_size(size_t size, request_rec *r) {
+    char ss[20];
+
     if(size == -1) 
-        rprintf (r, "    -");
+        strcpy(ss, "    -");
     else if(!size) 
-        rprintf (r, "   0K");
+        strcpy(ss, "   0k");
     else if(size < 1024) 
-        rprintf(r, "   1K");
+        strcpy(ss, "   1k");
     else if(size < 1048576)
-        rprintf(r, "%4dK", size / 1024);
+        sprintf(ss, "%4dk", size / 1024);
     else
-        rprintf(r, "%4dM", size / 1048576);
+        sprintf(ss, "%4dM", size / 1048576);
+    rputs(ss, r);
 }
 
+#ifdef __EMX__
+char **create_argv_cmd(pool *p, char *av0, char *args, char *path) {
+    register int x,n;
+    char **av;
+    char *w;
+
+    for(x=0,n=2;args[x];x++)
+        if(args[x] == '+') ++n;
+
+    /* Add extra strings to array. */
+    n = n + 2;
+
+    av = (char **)palloc(p, (n+1)*sizeof(char *));
+    av[0] = av0;
+
+    /* Now insert the extra strings we made room for above. */
+    av[1] = strdup("/C");
+    av[2] = strdup(path);
+
+    for(x=(1+2);x<n;x++) {
+        w = getword(p, &args, '+');
+        unescape_url(w);
+        av[x] = escape_shell_cmd(p, w);
+    }
+    av[n] = NULL;
+    return av;
+}
+#endif
+
